"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.makeVerify = void 0;
var errors_1 = require("./errors");
var types_1 = require("./types");
var isPresent = function (item) { return item !== undefined; };
var isBool = function (item) { return typeof item === "boolean"; };
var isNumber = function (item) { return typeof item === "number"; };
var isString = function (item) { return typeof item === "string"; };
var isArray = function (item) { return Array.isArray(item); };
var isTupleArray = function (item) {
    return Array.isArray(item) &&
        item.length > 0 &&
        item.every(function (it) { return isArray(it) && it.length === 2; });
};
var checkGetOptions = function (_a) {
    var options = _a.options;
    if (!isPresent(options))
        return types_1.VerifyResult.NoError;
    if (isPresent(options.raw) && !isBool(options.raw))
        return types_1.VerifyResult.RawOptionNotBool;
    if (isPresent(options.retainPath) && !isBool(options.retainPath))
        return types_1.VerifyResult.RetainPathOptionNotBool;
    if (isPresent(options.max_depth)) {
        if (!isNumber(options.max_depth))
            return types_1.VerifyResult.MaxDepthOptionNotNumber;
        if (options.max_depth < 0)
            return types_1.VerifyResult.MaxDepthOptionIsNegative;
    }
    return types_1.VerifyResult.NoError;
};
var checkSetValue = function (_a) {
    var path = _a.path, value = _a.value;
    if (isTupleArray(path))
        return types_1.VerifyResult.NoError;
    if (isArray(path)) {
        if (!isPresent(value) && !isTupleArray(path))
            return types_1.VerifyResult.ValueIsNotTupleArray;
        if (!isArray(value))
            return types_1.VerifyResult.ValueMustBeArray;
        if (path.length !== value.length)
            return types_1.VerifyResult.ValueAndPathArraysMustBeSameLength;
    }
    return value !== undefined && value !== null
        ? types_1.VerifyResult.NoError
        : types_1.VerifyResult.ValueIsMissing;
};
var checkAddValue = function (_a) {
    var path = _a.path, value = _a.value;
    if (isTupleArray(path))
        return types_1.VerifyResult.NoError;
    if (isArray(path)) {
        if (!isPresent(value)) {
            if (!isTupleArray(path))
                return types_1.VerifyResult.ValueIsNotTupleArray;
            else
                return types_1.VerifyResult.NoError;
        }
        if (!isArray(value))
            return types_1.VerifyResult.ValueMustBeArray;
        if (path.length !== value.length)
            return types_1.VerifyResult.ValueAndPathArraysMustBeSameLength;
    }
    return types_1.VerifyResult.NoError;
};
var checkSetOptions = function (_a) {
    var options = _a.options;
    if (!isPresent(options))
        return types_1.VerifyResult.NoError;
    if (isPresent(options.raw) && !isBool(options.raw))
        return types_1.VerifyResult.RawOptionNotBool;
    return types_1.VerifyResult.NoError;
};
var checkDelOptions = function (_a) {
    var allowPartial = _a.allowPartial;
    if (!isPresent(allowPartial))
        return types_1.VerifyResult.NoError;
    if (!isBool(allowPartial))
        return types_1.VerifyResult.AllowPartialOptionNotBool;
    return types_1.VerifyResult.NoError;
};
var checkSupportedDMOptions = function (_a) {
    var opts = _a.opts;
    if (!isPresent(opts))
        return types_1.VerifyResult.NoError;
    var allOptsAreBool = Object.values(opts).every(isBool);
    return allOptsAreBool ? types_1.VerifyResult.NoError : types_1.VerifyResult.OptionsMustBeBool;
};
var checkInstancesOptions = function (_a) {
    var opts = _a.opts;
    if (!isPresent(opts))
        return types_1.VerifyResult.NoError;
    if (isPresent(opts.firstLevelOnly) && !isBool(opts.firstLevelOnly))
        return types_1.VerifyResult.FirstLevelOnlyNotBool;
    return types_1.VerifyResult.NoError;
};
var checkProto = function (_a) {
    var proto = _a.proto;
    if (isPresent(proto) && !isString(proto))
        return types_1.VerifyResult.ProtoNotString;
    return types_1.VerifyResult.NoError;
};
var checkSetPaths = function (_a) {
    var path = _a.path, value = _a.value;
    if (!isPresent(path))
        return types_1.VerifyResult.PathsMissing;
    if (!isPresent(value) && isArray(path) && !isTupleArray(path))
        return types_1.VerifyResult.ValueIsNotTupleArray;
    return isString(path) ||
        (isArray(path) &&
            path.length > 0 &&
            path.every(function (path) { return typeof path === "string" || isArray(path); }))
        ? types_1.VerifyResult.NoError
        : types_1.VerifyResult.PathsNotStringOrStringArray;
};
var checkPaths = function (_a) {
    var initalPaths = _a.paths, initialPath = _a.path;
    var paths = initalPaths || initialPath;
    return !isPresent(paths)
        ? types_1.VerifyResult.PathsMissing
        : isString(paths) ||
            (isArray(paths) &&
                paths.length > 0 &&
                paths.every(function (path) { return typeof path === "string"; }))
            ? types_1.VerifyResult.NoError
            : types_1.VerifyResult.PathsNotStringOrStringArray;
};
var argsVerifiers = {
    GET: [checkPaths, checkGetOptions],
    SET: [checkSetPaths, checkSetValue, checkSetOptions],
    ADD: [checkSetPaths, checkAddValue, checkSetOptions],
    DELETE: [checkPaths, checkDelOptions],
    OPERATE: [checkPaths],
    GET_SUPPORTED_DM: [checkPaths, checkSupportedDMOptions],
    GET_INSTANCES: [checkPaths, checkInstancesOptions],
    GET_SUPPORTED_PROTO: [checkProto],
};
var translateError = function (errors, verifyResult) { return new Error((errors === null || errors === void 0 ? void 0 : errors[verifyResult]) || errors_1.defaultMissingErrorMessage); };
var makeVerify = function (errors) {
    return function (command, args) {
        var verifiers = argsVerifiers[command];
        if (!verifiers)
            return null;
        var results = verifiers.map(function (verifier) { return verifier(args); });
        var finalResult = results.find(function (value) { return value !== types_1.VerifyResult.NoError; }) ||
            types_1.VerifyResult.NoError;
        return finalResult === types_1.VerifyResult.NoError
            ? null
            : translateError(errors, finalResult);
    };
};
exports.makeVerify = makeVerify;
exports.default = exports.makeVerify;
